/*
 * Java port of Bullet (c) 2008 Martin Dvorak <jezek2@advel.cz>
 *
 * Bullet Continuous Collision Detection and Physics Library
 * Copyright (c) 2003-2007 Erwin Coumans  http://continuousphysics.com/Bullet/
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software.
 * 
 * Permission is granted to anyone to use this software for any purpose, 
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 * 
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software
 *    in a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 */

package javabullet.collision.shapes;

/**
 * QuantizedBvhNode is a compressed aabb node, 16 bytes.
 * Node can be used for leafnode or internal node. Leafnodes can point to 32-bit triangle index (non-negative range).
 * 
 * <p>JAVA NOTE: comments on memory are incorrect for Java
 * 
 * @author jezek2
 */
public class QuantizedBvhNode {

	public final /*unsigned*/ short[] quantizedAabbMin = new short[3];
	public final /*unsigned*/ short[] quantizedAabbMax = new short[3];
	public int escapeIndexOrTriangleIndex;
	
	public boolean isLeafNode() {
		// skipindex is negative (internal node), triangleindex >=0 (leafnode)
		return (escapeIndexOrTriangleIndex >= 0);
	}

	public int getEscapeIndex() {
		assert (!isLeafNode());
		return -escapeIndexOrTriangleIndex;
	}

	public int getTriangleIndex() {
		assert (isLeafNode());
		// Get only the lower bits where the triangle index is stored
		return (escapeIndexOrTriangleIndex & ~((~0) << (31 - OptimizedBvh.MAX_NUM_PARTS_IN_BITS)));
	}

	public int getPartId() {
		assert (isLeafNode());
		// Get only the highest bits where the part index is stored
		return (escapeIndexOrTriangleIndex >>> (31 - OptimizedBvh.MAX_NUM_PARTS_IN_BITS));
	}
	
}
